function f = nc_read(ncfile, Vname, Tindex, ReplaceValue, PreserveType)

%
% NC_READ:  Read requested NetCDF variable
%
% f = nc_read(ncfile, Vname, Tindex, ReplaceValue, PreserveType)
%
% This function reads in a generic variable from a NetCDF file. If only
% the water points are available, this function fill the land areas with
% zero and returns full fields.
%
% On Input:
%
%    ncfile        NetCDF file name or URL name (string)
%
%    Vname         NetCDF variable name to read (string)
%
%    Tindex        Optional, time record index to read (integer):
%                    If Tindex is provided, only the requested time record
%                    is read when the variable has the unlimited dimension
%                    or the word "time" in any of its dimension names.
%                    Otherwise, provide an empty [] argument.
%
%    ReplaceValue  Optional, value to use when _FillValue or missing_value
%                    attribute is found in variable. If not provided, a
%                    zero value will used.
%                    In some instances, like plotting, it is advantageous
%                    to set ReplaceValue = NaN to better visualize the
%                    land masking or the missing data.
%
%    PreserveType  Switch to preserve numerical data type. If false,
%                    convert numerical data to double precision. It
%                    has no effect on data that is already in double
%                    precision.
%
% On Output:
%
%    f             Field (scalar or array)
%
% calls:           nc_dinfo, nc_url, nc_vinfo, nc_Vname
%                  nc_info, nc_vargetr (SNCTOOLS java interface to OpenDAT)
%                  read_nc, read_nc_url (private functions below)
%

% svn $Id: nc_read.m 625 2012-07-03 20:07:22Z arango $
%=========================================================================%
%  Copyright (c) 2002-2012 The ROMS/TOMS Group                            %
%    Licensed under a MIT/X style license                                 %
%    See License_ROMS.txt                           Hernan G. Arango      %
%=========================================================================%

% If the PreserveType switch is not provided, set default to convert data
% to double precision.

if (nargin < 5),
  PreserveType = false;
end

% If ReplaceValue is not provided, use zero as a fill value.

if (nargin < 4),
  ReplaceValue = 0;
end

% If Tindex is not provided, use empty value.

if (nargin < 3)
  Tindex = [];
end

%--------------------------------------------------------------------------
% Read in requested variable.
%--------------------------------------------------------------------------

% Inquire about the contents of the NetCDF file.

Info = nc_inq(ncfile);

if (~any(strcmp({Info.Variables.Name}, Vname)))
  nc_inq(ncfile, true);
  disp(' ');
  error(['NC_READ: cannot find NetCDF variable: ',Vname]);
else
  ivar   = find(strcmp({Info.Variables.Name}, Vname));
  nvdims = length(Info.Variables(ivar).Dimensions);
end

PreserveType = true;
f = nc_read_matlab(ncfile,Vname,Tindex,ReplaceValue,PreserveType,Info);

%--------------------------------------------------------------------------
% If NetCDF file is a ROMS file with just the water data, build the full
% array (land and water points).
%--------------------------------------------------------------------------

if (nvdims > 0),
  if (any(strcmp({Info.Variables(ivar).Dimensions.Name}, 'xy_rho' )) || ...
      any(strcmp({Info.Variables(ivar).Dimensions.Name}, 'xyz_rho')));
    f = land_points (ncfile,Vname,ReplaceValue,Info,f);
  end
end

return

%--------------------------------------------------------------------------

function f = land_points (ncfile, Vname, ReplaceValue, Info, f)

%
% LAND_POINTS:  Adds land points to water data
%
% f = land_points(ncfile, Vname, ReplaceValue, Info, f)
%
% This function adds land points to read water data.  It build the full
% data array with land and water points.
%
% On Input:
%
%    ncfile        NetCDF file name or URL name (string)
%
%    Vname         NetCDF variable name to read (string)
%
%    ReplaceValue  Value to use when _FillValue or missing_value
%                    attribute is found in variable
%
%    Info          NetCDF information structure (struct array)
%
%    f             Water point data (1D array)
%
% On Output:
%
%    f             Full field including land and water points (array)
%

% Choose NetCDF file interface.

%[Fmethod,~,~] = nc_interface(ncfile);
  
% Inquire ROMS dimension values.

ndims = length(Info.Dimensions);

for n=1:ndims,
  dname =char(Info.Dimensions(n).Name);
  switch dname
    case 'xi_rho',
      Lr = Info.Dimensions(n).Length;
    case 'xi_u',
      Lu = Info.Dimensions(n).Length;
    case 'xi_v',
      Lv = Info.Dimensions(n).Length;
    case 'eta_rho',
      Mr = Info.Dimensions(n).Length;
    case 'eta_u',
      Mu = Info.Dimensions(n).Length;
    case 'eta_v',
      Mv = Info.Dimensions(n).Length;
    case 's_rho',
      Nr = Info.Dimensions(n).Length;
    case 's_w',
      Nw = Info.Dimensions(n).Length;
  end
end  
 
% Check if data is only available at water points.

is2d = false;
is3d = false;

ivar = find(strcmp({Info.Variables.Name}, Vname));

if (~isempty(ivar)),
  nvdims = length(Info.Variables(ivar).Dimensions);

  for n=1:nvdims,
    dname = char(Info.Variables(ivar).Dimensions(n).Name);
    switch dname
      case 'xy_rho',
        msknam = 'mask_rho';
        is2d = true; Im = Lr; Jm = Mr;
      case 'xy_u',
        msknam = 'mask_u';
        is2d = true; Im = Lu; Jm = Mu;
      case 'xy_v',
        msknam = 'mask_v';
        is2d = true; Im = Lv; Jm = Mv;
      case 'xyz_rho',
        msknam = 'mask_rho';
        is3d = true; Im = Lr; Jm = Mr; Km = Nr;
      case 'xyz_u',
        msknam = 'mask_u';
        is3d = true; Im = Lu; Jm = Mu; Km = Nr;
      case 'xyz_v',    
        msknam='mask_v';
        is3d = true; Im = Lv; Jm = Mv; Km = Nr;
      case 'xyz_w',    
        msknam = 'mask_rho';
        is3d = true; Im = Lr; Jm = Mr; Km = Nw;
    end
  end
end

water = is2d | is3d;

% If water data only, read in the appropriate Land/Sea mask.

if (water),
  got_mask = any(strcmp(vnames,msknam));

  if (got_mask)
      mask = nc_read_matlab(ncfile,msknam,[],ReplaceValue,false,Info);
  else
      Gname = input('Enter grid NetCDF file: ');
      mask = nc_read_matlab(Gname,msknam,[],ReplaceValue,false,Info);
  end
  
end

% Build full array with land and water points.

if (water),

  v = f;
  [Npts,Ntime]=size(v);

  if (is2d),
    f = squeeze(ones([Im,Jm,Ntime])).*ReplaceValue;
    MASK = squeeze(repmat(mask,[1,1,Ntime]));
    ind = MASK > 0;
    f(ind) = v;
  elseif (is3d),    
    f = squeeze(ones([Im,Jm,Km,Ntime])).*ReplaceValue;
    MASK = squeeze(repmat(mask,[1,1,Km,Ntime]));
    ind = MASK > 0;
    f(ind) = v;
  end

end

return

%--------------------------------------------------------------------------

function f = nc_read_matlab(ncfile, Vname, Tindex,ReplaceValue, PreserveType, Info)

%
% NC_READ_MATLAB:  Internal routine to read requested NetCDF variable
%
% f = nc_read_matlab(ncfile,Vname,Tindex,ReplaceValue,PreserveType,Info)
%
% This function reads in requested variable using Matlab's native NetCDF
% inteface.
%
% On Input:
%
%    ncfile        NetCDF file name or URL name (character string)
%
%    Vname         NetCDF variable name to read (character string)
%
%    Tindex        Time record index to read (integer)
%
%    ReplaceValue  Value to use when "_FillValue" or "missing_value"
%                    attribute is found in variable.
%
%    PreserveType  Switch to preserve numerical data type. If false,
%                    convert numerical data to double precision. It
%                    has not effect data is already in double
%                    precision.
%
%    Info          NetCDF information structure (struct array)
%
% On Output:
%
%    f             Field (scalar or array)
%

% Initialize.

got.add_offset    = false;
got.FillValue     = false;
got.missing_value = false;
got.RecDim        = false;
got.scale_factor  = false;

ivar   = find(strcmp({Info.Variables.Name}, Vname));
nvdims = length(Info.Variables(ivar).Dimensions);
nvatts = length(Info.Variables(ivar).Attributes);
nctype = Info.Variables(ivar).ncType;

% Activate switch for reading specific record.

time_rec = false;
if (~isempty(Tindex)),
  time_rec = true;
end,

% Check if there is an unlimited dimension or a time dimension.

if (nvdims > 0),
  for n=1:nvdims,
    dname = char(Info.Variables(ivar).Dimensions(n).Name);
    if (Info.Variables(ivar).Dimensions(n).Unlimited ||                 ...
        ~isempty(strfind(dname,'time'))),
      got.RecDim = true;
      TimeDimName = dname;
    end
  end
end

% Inquire information about the attributes.

if (nvatts > 0),
  for n=1:nvatts,
    aname = char(Info.Variables(ivar).Attributes(n).Name);
    switch aname
      case 'add_offset'
        offset = Info.Variables(ivar).Attributes(n).Value;
        got.add_offset = true;
      case {'_FillValue', '_fillvalue', 'missing_value'}
        spval = Info.Variables(ivar).Attributes(n).Value;
        if (strcmp(aname, 'missing_value')),
          got.missing_value = true;
        else
          got.FillValue = true;
        end
      case 'scale_factor'
        scale = Info.Variables(ivar).Attributes(n).Value;
        got.scale_factor = true;
    end
  end
end

% Set start and count indices to process.

if (nvdims > 0),
  start = zeros([1 nvdims]);
  count = Inf([1 nvdims]); 

  for n=1:nvdims,
    dname = char(Info.Variables(ivar).Dimensions(n).Name);
    start(n) = 0;
    count(n) = Info.Variables(ivar).Dimensions(n).Length;
    if (time_rec && got.RecDim),
      if (strcmp(dname, TimeDimName)),
        start(n) = Tindex-1;
        count(n) = 1;
      end
    end
  end
end

%--------------------------------------------------------------------------
% Read in requested variable. Use SNCTOOLS function "nc_vargetr" to read
% variable in its native (raw) data type.
%--------------------------------------------------------------------------

% Open NetCDF file.

ncid  = netcdf.open(ncfile, 'nc_nowrite');
varid = netcdf.inqVarID(ncid, Vname);

% Read in data: scalar or multi-dimensional array.

if (nvdims == 0),
  f = netcdf.getVar(ncid, varid);
else
  f = netcdf.getVar(ncid, varid, start, count);
end

% Close NetCDF file.

netcdf.close(ncid);

%--------------------------------------------------------------------------
% Post-process read data.
%--------------------------------------------------------------------------

% Search for fill value or missing values.

ind = [];

if (got.FillValue || got.missing_value),
  if (iscellstr(f) || ischar(f)),
    ind = find(f == spval);
    f(ind) = spval;
  else
    ind = find(abs(f-spval) < 4*eps(double(spval)));
  end
end

% Scale data and/or add offset value.

if (isnumeric(f)),
  if (got.add_offset),
    if (got.scale_factor),
      switch nctype
       case {netcdf.getConstant('nc_float'),                            ...
             netcdf.getConstant('nc_double')}
          f = f.*scale+offset;
        case {nc_int, nc_short, nc_byte}
          f = double(f).*scale+offset;
      end
    else
      switch nctype
        case {netcdf.getConstant('nc_float'),                           ...
              netcdf.getConstant('nc_double')}
          f = f+offset;
        case {netcdf.getConstant('nc_int'),                             ...
              netcdf.getConstant('nc_short'),                           ...
              netcdf.getConstant('nc_byte')}
          f = double(f)+offset;
      end
    end
  elseif (got.scale_factor);
    switch nctype
      case {netcdf.getConstant('nc_float'),                             ...
            netcdf.getConstant('nc_double')}
        f = f.*scale;
      case {netcdf.getConstant('nc_int'),                               ...
            netcdf.getConstant('nc_short'),                             ...
            netcdf.getConstant('nc_byte')}
        f = double(f).*scale;
    end
  end
end

% Set fill values or missing values with specified ReplaceValue.

if (~isempty(ind) && isnumeric(f)),
  f(ind) = ReplaceValue;
end

%--------------------------------------------------------------------------
% If requested and applicable, convert data to double precision.
%--------------------------------------------------------------------------

if (~PreserveType) && ~(iscellstr(f) || ischar(f)),
  f = double(f);
end

return
