function status = nc_write(ncfile, Vname, f, varargin)

%
% NC_WRITE:  Writes a variable into a NetCDF file
%
% status = nc_write(ncfile, Vname, f, Tindex)
%
% This routine writes in a generic variable into a NetCDF file.
%
% On Input:
%
%    ncfile      NetCDF file name (string)
%
%    Vname       NetCDF variable name to read (string)
%
%    f           Field (scalar, matrix or array)
%
%    Tindex      Optional, time index to write (integer):
%
%                  *  If tindex is not provided as an argument during
%                     function call, it is assumed that the entire
%                     variable is to be written. 
%
%                  *  If variable has an unlimited record dimension,
%                     tindex can be used to increase that dimension or
%                     replace an already existing record.
%
%                  *  If variable has the word "time" in its dimension
%                     name, tindex can be used to write at the specified
%                     the time record.
%
% On Output:
%
%    status      Error flag
%

% svn $Id: nc_write.m 625 2012-07-03 20:07:22Z arango $
%=========================================================================%
%  Copyright (c) 2002-2012 The ROMS/TOMS Group                            %
%    Licensed under a MIT/X style license                                 %
%    See License_ROMS.txt                           Hernan G. Arango      %
%=========================================================================%

% Initialize.

Tindex = [];
switch numel(varargin)
  case 1
    Tindex = varargin{1};
end
  
% Inquire about the contents of the NetCDF file.

Info = nc_inq(ncfile);

if (~any(strcmp({Info.Variables.Name}, Vname)))
  nc_inq(ncfile, true);
  disp(' ');
  error(['NC_WRITE: cannot find NetCDF variable: ',Vname]);
end

% Choose NetCDF file interface.
status = nc_write_matlab(ncfile, Vname, f, Tindex, Info);

return

%--------------------------------------------------------------------------

function status = nc_write_matlab(ncfile, Vname, f, Tindex, Info)

%
% NC_WRITE_MATLAB:  Writes a variable into a NetCDF file
%
% status = nc_write(ncfile, Vname, f, Tindex, Info)
%
% This routine writes the specified variable into a NetCDF file using the
% native interface.
%
% On Input:
%
%    ncfile      NetCDF file name (string)
%
%    Vname       NetCDF variable name to read (string)
%
%    f           Field (scalar or array)
%
%    Tindex      Optional, time index to write:
%
%                  *  If Tindex is not provided as an argument during
%                     function call, it is assumed that entire variable
%                     is to be written. 
%
%                  *  If variable has an unlimited record dimension,
%                     Tindex can be used to increase that dimension or
%                     replace an already existing record.
%
%                  *  If variable has the word "time" in its dimension
%                     name, Tindex can be use to write at the specified
%                     the time record.
%
%    Info        NetCDF information structure (struct array)
%
% On Output:
%
%    status      Error flag
%

% Initialize.

if (isempty(Tindex))
  time_rec = false;
else
  time_rec = true;
end

got.FillValue     = false;
got.missing_value = false;
got.RecDim        = false;

ivar   = find(strcmp({Info.Variables.Name}, Vname));
nvdims = length(Info.Variables(ivar).Dimensions);
nvatts = length(Info.Variables(ivar).Attributes);
nctype = Info.Variables(ivar).ncType;

status = 0;

% Check if there is an unlimited dimension or a time dimension.

index = 0;
if (nvdims > 0),
  for n=1:nvdims,
    dname = char(Info.Variables(ivar).Dimensions(n).Name);
    if (Info.Variables(ivar).Dimensions(n).Unlimited ||                 ...
        ~isempty(strfind(dname,'time'))),
      got.RecDim = true;
      index = n;
    end
  end
end

% Inquire information about the attributes.

if (nvatts > 0),
  for n=1:nvatts,
    aname = char(Info.Variables(ivar).Attributes(n).Name);
    switch aname
      case {'_FillValue', '_fillvalue', 'missing_value'}
        spval = Info.Variables(ivar).Attributes(n).Value;
        if (strcmp(aname, 'missing_value')),
          got.missing_value = true;
        else
          got.FillValue = true;
        end
    end
  end
end

% Set start and count indices to process.

if (nvdims > 0),
  start = zeros([1 nvdims]);
  count = Inf([1 nvdims]); 

  for n=1:nvdims,
    start(n) = 0;
    count(n) = Info.Variables(ivar).Dimensions(n).Length;
  end
end

%  It writing specific time record, reset variable bounds.  Check
%  for files having more than one datum in the unlimited dimension
% (like variational data assimilation files).

datum = false;
if (got.RecDim  && (nvdims == 1)),
  if (length(f) > 1)
    datum = true;
    if (time_rec),
      start(index) = Tindex-1;
    else
      start(index) = 0;
    end
    count(index) = length(f);
  end
end

if (~datum && time_rec && (index > 0)),
  start(index) = Tindex-1;
  count(index) = 1;
end

%--------------------------------------------------------------------------
% If _FillValue attribute, replace NaNs with fill value.
%--------------------------------------------------------------------------

% Compute the minimum and maximum of the data to write.

fmin = min(f(:));
fmax = max(f(:));

% Replace NaNs if any with fill value.

if (got.FillValue),
  ind = isnan(f);
  if (~isempty(ind))
    switch nctype
      case (netcdf.getConstant('nc_double'))
        f(ind) = double(spval);
      case (netcdf.getConstant('nc_float'))
        f(ind) = single(spval);
      case (netcdf.getConstant('nc_int'))
        f(ind) = int32(spval);
      case (netcdf.getConstant('nc_short'))
        f(ind) = int16(spval);
      case (netcdf.getConstant('nc_byte'))
        f(ind) = int8(spval);
      otherwise
        f(ind) = spval;
    end
  end
end

%--------------------------------------------------------------------------
% Write out variable into NetCDF file.
%--------------------------------------------------------------------------

% Open NetCDF file and get variable ID.

ncid  = netcdf.open(ncfile, 'nc_write');
varid = netcdf.inqVarID(ncid, Vname);

% Write out data.

if (nvdims > 0),
  netcdf.putVar(ncid, varid, start, count, f)
else
  netcdf.putVar(ncid, varid, f)
end  

% Close NetCDF file.

netcdf.close(ncid);

% Report.

if (nvdims > 1),
  text(1:19)=' ';
  text(1:length(Vname))=Vname;
  if (nargin > 3),
    disp(['Wrote ',sprintf('%19s',text),                                ...
          ' into record: ',num2str(Tindex,'%4.4i'),                     ...
          ', Min=',sprintf('%12.5e',fmin),                              ...
          ' Max=',sprintf('%12.5e',fmax)]);
  else
    disp(['Wrote ',sprintf('%19s',text),                                ...
          ' Min=',sprintf('%12.5e',fmin),                               ...
          ' Max=',sprintf('%12.5e',fmax)]);
  end
end

return

%--------------------------------------------------------------------------

